@@ -1,5 +1,3 @@
-use std::time::UNIX_EPOCH;
-
 use proc_macro::TokenStream;
 use proc_macro2::TokenStream as TokenStream2;
 use quote::{format_ident, quote, ToTokens};
@@ -15,6 +13,7 @@ struct MessageArgs {
     ret: Option<Type>,
     part: bool,
     transparent: bool,
+    dumping_allowed: bool,
     crate_: Path,
     not: Vec<String>,
 }
@@ -26,6 +25,7 @@ impl Parse for MessageArgs {
             name: None,
             part: false,
             transparent: false,
+            dumping_allowed: true,
             crate_: parse_quote!(::elfo),
             not: Vec::new(),
         };
@@ -37,6 +37,7 @@ impl Parse for MessageArgs {
         // `#[message(part, transparent)]`
         // `#[message(elfo = some)]`
         // `#[message(not(Debug))]`
+        // `#[message(dumping = "disabled")]`
         while !input.is_empty() {
             let ident: Ident = input.parse()?;
 
@@ -51,6 +52,19 @@ impl Parse for MessageArgs {
                 }
                 "part" => args.part = true,
                 "transparent" => args.transparent = true,
+                "dumping" => {
+                    // TODO: introduce `DumpingMode`.
+                    let _: Token![=] = input.parse()?;
+                    let s: LitStr = input.parse()?;
+
+                    assert_eq!(
+                        s.value(),
+                        "disabled",
+                        "only `dumping = \"disabled\"` is supported"
+                    );
+
+                    args.dumping_allowed = false;
+                }
                 // TODO: call it `crate` like in linkme?
                 "elfo" => {
                     let _: Token![=] = input.parse()?;
@@ -77,12 +91,6 @@ impl Parse for MessageArgs {
     }
 }
 
-fn gen_ltid() -> u32 {
-    // TODO
-    let elapsed = UNIX_EPOCH.elapsed().expect("invalid system time");
-    elapsed.as_nanos() as u32
-}
-
 fn gen_derive_attr(blacklist: &[String], name: &str, path: TokenStream2) -> TokenStream2 {
     let tokens = if blacklist.iter().all(|x| x != name) {
         quote! { #[derive(#path)] }
@@ -132,7 +140,6 @@ pub fn message_impl(args: TokenStream, input: TokenStream) -> TokenStream {
     let input = parse_macro_input!(input as DeriveInput);
     let name = &input.ident;
     let mod_name = format_ident!("_elfo_{}", name);
-    let ltid = gen_ltid();
     let serde_crate = format!("{}::_priv::serde", args.crate_.to_token_stream());
     let crate_ = args.crate_;
     let internal = quote![#crate_::_priv];
@@ -223,16 +230,13 @@ pub fn message_impl(args: TokenStream, input: TokenStream) -> TokenStream {
         quote! {}
     };
 
+    // TODO: pass to `Wrapper`.
+    let dumping_allowed = args.dumping_allowed;
+
     let impl_message = if !args.part {
         quote! {
             impl #crate_::Message for #name {
-                const _LTID: #internal::LocalTypeId = #ltid;
-                const PROTOCOL: &'static str = #protocol;
-                const NAME: &'static str = #name_str;
-                const LABELS: &'static [#internal::metrics::Label] = &[
-                    #internal::metrics::Label::from_static_parts("message", Self::NAME),
-                    #internal::metrics::Label::from_static_parts("protocol", Self::PROTOCOL),
-                ];
+                const VTABLE: &'static #internal::MessageVTable = #mod_name::VTABLE;
 
                 #[inline(always)]
                 fn _touch(&self) {
@@ -250,7 +254,7 @@ pub fn message_impl(args: TokenStream, input: TokenStream) -> TokenStream {
 
                 use #internal::{
                     MESSAGE_LIST, MessageVTable, AnyMessage,
-                    smallbox::smallbox, linkme
+                    smallbox::smallbox, linkme, metrics,
                 };
 
                 #ret_wrapper
@@ -271,18 +275,23 @@ pub fn message_impl(args: TokenStream, input: TokenStream) -> TokenStream {
                     smallbox!(Clone::clone(cast_ref(message)))
                 }
 
-                #[linkme::distributed_slice(MESSAGE_LIST)]
-                #[linkme(crate = #internal::linkme)]
-                static VTABLE: MessageVTable = MessageVTable {
-                    ltid: #name::_LTID,
-                    name: #name::NAME,
-                    protocol: #name::PROTOCOL,
-                    labels: #name::LABELS,
+                pub(crate) const VTABLE: &'static MessageVTable = &MessageVTable {
+                    name: #name_str,
+                    protocol: #protocol,
+                    labels: &[
+                        metrics::Label::from_static_parts("message", #name_str),
+                        metrics::Label::from_static_parts("protocol", #protocol),
+                    ],
+                    dumping_allowed: #dumping_allowed,
                     clone,
                     debug,
                     erase,
                 };
 
+                #[linkme::distributed_slice(MESSAGE_LIST)]
+                #[linkme(crate = linkme)]
+                static VTABLE_STATIC: &'static MessageVTable = #name::VTABLE;
+
                 // See [rust#47384](https://github.com/rust-lang/rust/issues/47384).
                 #[doc(hidden)]
                 #[inline(never)]
